Unit Profile;

Interface

Uses OPCrt,Dos;

Type
  TCartName = String[12];
  TCart = (ct2k,ct4k,ct8k,ct16k,ctSuperChip,ctParkerBros,ctMNetwork,
           ctStarpath,ctCBS,ctTigerVision,ctPitfall2,ctActivision,
           ctMegaBoy,ct32In1,ctUnknown);
  TController = (crJoystick,crPaddle,crVTP,crDriving,crKeyboard,crUnknown);

Const
  MenuKey : Array[TCart] Of Char =
   (#13,#13,#13,#13,'R','P','M',#13,#13,'T','2','A','G','3',#13);

Procedure MakeProfile(Path: String);
Function  FindProfile(Name: TCartName; Var Controller: TController): TCart;

Implementation

Const
  InFileName  = 'PCAE.KNW';
  OutFileName = 'PCAE.PRO';

Type
  PFileType = ^TFileType;
  TFileType = Record
    Name : TCartName;
    Cart : TCart;
    Cont : TController;
    Next : PFileType;
  End;
  PIndex = ^TIndex;
  TIndex = Array[0..255] Of Pointer;

Const
  CartID : Array[TCart] Of String[2] =
   ('2K','4K','8K','16','SC','PB','MN','SP','CB','TV','P2','AC','MB','32','??');
  ContID : Array[TController] Of String[2] =
   ('JY','PD','VT','DR','KB','%%');

Var
  FHead       : PFileType;
  FTail       : PFileType;
  Buf         : Array[0..255] Of Byte;
  OldExitProc : Pointer;
  FileIndex   : PIndex;

Function NoSpace(St: String): String;
Begin
  While (St <> '') And (St[1]          In [#9,' ']) Do St := Copy(St,2,Length(St) - 1);
  While (St <> '') And (St[Length(St)] In [#9,' ']) Do St := Copy(St,1,Length(St) - 1);
  NoSpace := St;
End; { NoSpace }

Function UpperCase(St: String): String;
Var I: Integer;
Begin
  For I := 1 To Length(St) Do St[I] := UpCase(St[I]);
  UpperCase := St;
End; { UpperCase }

Function Exist(FileName: String): Boolean;
Var Fil: File;
Begin
  Assign(Fil,FileName);
  {$I-}
  Reset(Fil);
  {$I+}
  If (IOResult = 0) Then
  Begin
    Close(Fil);
    Exist := True;
  End
  Else Exist := False;
End; { Exist }

Procedure SortFileList;
Var
  F1,F2 : PFileType;
  St    : TCartName;
  Cart  : TCart;
  Cont  : TController;

Begin
  WriteLn('Sorting file names...');
  F1 := FHead;
  While F1^.Next <> Nil Do
  Begin
    F2 := F1^.Next;
    While F2 <> Nil Do
    Begin
      If F2^.Name < F1^.Name Then
      Begin
        St       := F1^.Name;
        F1^.Name := F2^.Name;
        F2^.Name := St;
        Cart     := F1^.Cart;
        F1^.Cart := F2^.Cart;
        F2^.Cart := Cart;
        Cont     := F1^.Cont;
        F1^.Cont := F2^.Cont;
        F2^.Cont := Cont;
      End;
      F2 := F2^.Next;
    End; { While }
    F1 := F1^.Next;
  End;
End; { SortFileList }

Procedure LoadFileList(Path: String);
Var S: SearchRec;
Begin
  WriteLn('Loading file names...');
  FindFirst(Path + '*.BIN',Archive,S);
  While DOSError = 0 Do
  Begin
    If FHead = Nil Then
    Begin
      New(FHead);
      FTail := FHead;
    End
    Else
    Begin
      New(FTail^.Next);
      FTail := FTail^.Next;
    End;
    FTail^.Name := UpperCase(S.Name);
    FTail^.Cart := ctUnknown;
    FTail^.Cont := crUnknown;
    FTail^.Next := Nil;
    FindNext(S);
  End; { While }
  FindFirst(Path + '*.PAL',Archive,S);
  While DOSError = 0 Do
  Begin
    If FHead = Nil Then
    Begin
      New(FHead);
      FTail := FHead;
    End
    Else
    Begin
      New(FTail^.Next);
      FTail := FTail^.Next;
    End;
    FTail^.Name := UpperCase(S.Name);
    FTail^.Cart := ctUnknown;
    FTail^.Cont := crUnknown;
    FTail^.Next := Nil;
    FindNext(S);
  End; { While }
End; { LoadFileList }

Procedure Setup;
Begin
  FHead := Nil;
  FTail := Nil;
End; { Setup }

Procedure Cleanup;
Var FW1,FW2: PFileType;
Begin
  FW1 := FHead;
  While FW1 <> Nil Do
  Begin
    FW2 := FW1^.Next;
    Dispose(FW1);
    FW1 := FW2;
  End; { While }
  FHead := Nil;
  FTail := Nil;
  FillChar(FileIndex^,SizeOf(FileIndex^),#0);
End; { Cleanup }

Procedure OutputFile(Path: String);
Var
  F       : PFileType;
  Loaded  : LongInt;
  Load    : LongInt;
  Cart    : TCart;
  B       : Byte;
  I       : Integer;
  Done    : Boolean;
  St      : String;
  C       : Char;
  Lines   : Integer;
  InFile  : File;
  OutFile : Text;

Begin
  C := #0;
  Write('Outputting... ');
  Assign(OutFile,OutFileName);
  ReWrite(OutFile);
  WriteLn(OutFile,'; PCAE cartridge profile list');
  WriteLn(OutFile,'; Valid cart codes: 2K 4K 8K 16 SC PB MN SP CB TV P2 AC MB 32 ??');
  WriteLn(OutFile,'; Valid controller codes: JY PD VT DR KB %%');
  WriteLn(OutFile);
  F     := FHead;
  Lines := 0;
  While F <> Nil Do
  Begin
    If F^.Name[1] <> C Then
    Begin
      C := F^.Name[1];
      Write(C,' ');
    End;
    Assign(InFile,Path + F^.Name);
    Reset(InFile,1);
    Loaded := FileSize(InFile);
    Cart   := ctUnknown;
    If Loaded < 4096 Then Cart := ct2k Else
     If Loaded = 4096 Then Cart := ct4k Else
      If Loaded = 8192 Then Cart := ct8k Else
       If (Loaded Mod 8448) = 0 Then Cart := ctStarpath Else
        If Loaded = 12288 Then Cart := ctCBS Else
         If Loaded = 16384 Then Cart := ct16k Else
          If Loaded > 16384 Then Cart := ctSuperChip;
    If Loaded > 256 Then Load := 256 Else Load := Loaded;
    BlockRead(InFile,Buf,Load);
    Close(InFile);

    If {(Loaded = 8192) Or} (Loaded = 10495) Then
    Begin
      B    := Buf[0];
      Done := False;
      I := 0;
      While (I < 128) And Not Done Do
      Begin
        If Buf[I] <> B Then Done := True;
        Inc(I);
      End; { While }
      If Not Done Then Cart := ctPitfall2;
    End;
    If Loaded = 16384 Then
    Begin
      B := Buf[0];
      Done := False;
      I := 0;
      While (I < 256) And Not Done Do
      Begin
        If Buf[I] <> B Then Done := True;
        Inc(I);
      End; { While }
      If Not Done Then Cart := ctSuperChip;
    End;
    If F^.Cart = ctUnknown Then F^.Cart := Cart;
    St := F^.Name;
    While Length(St) < 15 Do St := St + ' ';
    I  := 3 * Integer(F^.Cart);
    While I > 0 Do
    Begin
      St := St + ' ';
      Dec(I);
    End; { While }
    St := St + CartID[F^.Cart] + ' ' + ContID[F^.Cont];
    WriteLn(OutFile,St);
    F := F^.Next;
    Inc(Lines);
    If (Lines Mod 10) = 0 Then
     WriteLn(OutFile,'; ---------------------------------------------------');
  End; { While }
  Close(OutFile);
  WriteLn;
End; { OutputFile }

Procedure LoadKnownList;
Var
  St,St1  : String;
  I       : Integer;
  F       : PFileType;
  Found   : Boolean;
  Cart    : TCart;
  Cont    : TController;
  OutFile : Text;

Begin
  WriteLn('Loading known cartridge profile list...');
  If Exist(InFileName) Then
  Begin
    Assign(OutFile,InFileName);
    Reset(OutFile);
    While Not Eof(OutFile) Do
    Begin
      ReadLn(OutFile,St);
      St := UpperCase(NoSpace(St));
      If (St <> '') And (St[1] <> ';') Then
      Begin
        I  := Pos(' ',St);
        If I <> 0 Then
        Begin
          St1   := NoSpace(Copy(St,1,I - 1));
          St    := NoSpace(Copy(St,I + 1,Length(St) - I));
          F     := FHead;
          Found := False;
          While (F <> Nil) And Not Found Do
          Begin
            If St1 = F^.Name Then Found := True Else F := F^.Next;
          End; { While }
          If Found Then
          Begin
            I := Pos(' ',St);
            If I <> 0 Then
            Begin
              St1 := NoSpace(Copy(St,I + 1,Length(St) - I));
              St  := NoSpace(Copy(St,1,I - 1));
            End
            Else St1 := '  ';
            For Cart := ct2k To ctUnknown Do
             If (CartID[Cart] = St) Or (CartID[Cart] = St1) Then F^.Cart := Cart;
            For Cont := crJoystick To crUnknown Do
             If (ContID[Cont] = St) Or (ContID[Cont] = St1) Then F^.Cont := Cont;
{
            For Cart := ct2k To ctUnknown Do
             If CartID[Cart] = St Then F^.Cart := Cart;
}
          End;
        End;
      End;
    End; { While }
    Close(OutFile);
  End;
End; { LoadKnownList }

Procedure IndexFileList;
Var
  FWork : PFileType;
  C     : Char;

Begin
  FillChar(FileIndex^,SizeOf(FileIndex^),#0);
  FWork := FHead;
  C     := '*';   { Choose a character that cannot be in the filename }
  While FWork <> Nil Do
  Begin
    If FWork^.Name <> '' Then
    Begin
      If FWork^.Name[1] <> C Then
      Begin
        C                  := FWork^.Name[1];
        FileIndex^[Ord(C)] := FWork;
      End;
    End;
    FWork := FWork^.Next;
  End; { While }
End; { IndexFileList }

Procedure MakeProfile;
Begin
  Cleanup;
  Setup;
  LoadFileList(Path);
  SortFileList;
  LoadKnownList;
  IndexFileList;
  OutputFile(Path);
End; { MakeProfile }

Procedure LoadProfile;
Var
  St,St1  : String;
  I       : Integer;
  F       : PFileType;
  Found   : Boolean;
  Cart    : TCart;
  Cont    : TController;
  OutFile : Text;

Begin
  If Exist(OutFileName) Then
  Begin
    WriteLn('Loading cartridge profile...');
    Assign(OutFile,OutFileName);
    Reset(OutFile);
    While Not Eof(OutFile) Do
    Begin
      ReadLn(OutFile,St);
      St := UpperCase(NoSpace(St));
      If (St <> '') And (St[1] <> ';') Then
      Begin
        I  := Pos(' ',St);
        If I <> 0 Then
        Begin
          St1   := NoSpace(Copy(St,1,I - 1));
          St    := NoSpace(Copy(St,I + 1,Length(St) - I));
          If FHead = Nil Then
          Begin
            New(FHead);
            FTail := FHead;
          End
          Else
          Begin
            New(FTail^.Next);
            FTail := FTail^.Next;
          End;
          FTail^.Name := St1;
          FTail^.Next := Nil;
          FTail^.Cart := ctUnknown;
          FTail^.Cont := crUnknown;
          I := Pos(' ',St);
          If I <> 0 Then
          Begin
            St1 := NoSpace(Copy(St,I + 1,Length(St) - I));
            St  := NoSpace(Copy(St,1,I - 1));
          End
          Else St1 := '  ';
          For Cart := ct2k To ctUnknown Do
           If (CartID[Cart] = St) Or (CartID[Cart] = St1) Then FTail^.Cart := Cart;
          For Cont := crJoystick To crUnknown Do
           If (ContID[Cont] = St) Or (ContID[Cont] = St1) Then FTail^.Cont := Cont;
        End;
      End;
    End; { While }
    Close(OutFile);
  End;
End; { LoadProfile }

{$F+}
Procedure KillProfile;
Begin
  Cleanup;
  Dispose(FileIndex);
  ExitProc := OldExitProc;
End; { KillProfile }
{$F-}

Function FindProfile(Name: TCartName; Var Controller: TController): TCart;
Var
  FWork : PFileType;
  Found : Boolean;

Begin
  Name  := UpperCase(Name);
  Found := False;
  If Name <> '' Then
  Begin
    FWork := FileIndex^[Ord(Name[1])];
    While (FWork <> Nil) And (FWork^.Name[1] = Name[1]) And Not Found Do
    Begin
      If Name = FWork^.Name Then Found := True Else FWork := FWork^.Next;
    End; { While }
  End;
  If Found Then
  Begin
    FindProfile := FWork^.Cart;
    Controller  := FWork^.Cont;
  End
  Else
  Begin
    FindProfile := ctUnknown;
    Controller  := crUnknown;
  End;
End; { FindProfile }

Begin
  FHead := Nil;
  FTail := Nil;
  New(FileIndex);
  LoadProfile;
  IndexFileList;
  OldExitProc := ExitProc;
  ExitProc    := @KillProfile;
End.
